@@ -148,25 +148,27 @@ namespace XMLImplementation{
 	/*
 	 * Constructor from class XMLHandler, initializes varibles
 	 */
-	XMLHandler::XMLHandler(string dir, shared_ptr<Activity> activity, shared_ptr<LogError> errors, shared_ptr<DBImages> images):m_pLocator(0){
-			m_activitat = activity;
-			m_errors = errors;
-			m_image = shared_ptr<Picture>();
-			m_mod = shared_ptr<Module>();
-			m_defTrIn = typeTransition(0);
-			m_defTrOut = typeTransition(0);
-			m_errorSrc = false;
-			m_trans = false;
-			m_dir = dir;
-			m_degrees = 90;
-			m_errorKrnl = false;
-			m_def = false;
-			m_x = 0.0f;
-			m_y = 0.0f;
-			m_images = images;
-			m_level = 0;
-			m_tagError = false;
+	XMLHandler::XMLHandler(string dir, shared_ptr<Activity> activity, shared_ptr<LogError> errors, shared_ptr<DBImages> images)
+	: m_pLocator(0)
+	{
+		m_activitat = activity;
+		m_errors = errors;
+		m_image = shared_ptr<Picture>();
+		m_mod = shared_ptr<Module>();
+		m_defTrIn= m_defTrOut= m_trIn= m_trOut= Pictures::NIL;
+		m_errorKrnl = false;
+		m_errorSrc = false;
+		m_trans = false;
+		m_def = false;
+		m_tagError = false;
+		m_level = 0;
+		m_dir = dir;
+		m_degrees = 90;		
+		m_x = 0.0f;
+		m_y = 0.0f;
+		m_images = images;		
 	}
+
 	/*
 	 * Destructor from class XMLHandler
 	 */
