@@ -43,6 +43,7 @@
 #include <iostream>
 
 using namespace spcore;
+using namespace std;
 
 namespace mod_puredata {
 
@@ -128,7 +129,7 @@ wxWindow* PlayWithVoiceComponent::GetGUI(wxWindow * parent)
 	return m_panel;
 }
 
-int PlayWithVoiceComponent::Start()
+int PlayWithVoiceComponent::DoStart()
 {
 	if (m_running) return 0;
 
@@ -166,7 +167,7 @@ int PlayWithVoiceComponent::Start()
 	return 0;
 }
 
-void PlayWithVoiceComponent::Stop()
+void PlayWithVoiceComponent::DoStop()
 {
 	if (!m_running) return;
 
