@@ -46,13 +46,13 @@
 ////@end XPM images
 
 using namespace spcore;
+using namespace std;
 
 namespace mod_puredata {
 
 PureDataConfigComponent::PureDataConfigComponent(const char * name, int argc, const char * argv[])
 : CComponentAdapter(name, argc, argv)
 , m_pdError(false)
-, m_initialized(false)
 , m_panel(NULL)
 , m_oscIn(50003, this)
 , m_oscOut(50002)
@@ -125,10 +125,8 @@ wxWindow* PureDataConfigComponent::GetGUI(wxWindow * parent)
 
 
 	
-int PureDataConfigComponent::Initialize()
+int PureDataConfigComponent::DoInitialize()
 {
-	if (m_initialized) return 0;
-
 	try {
 		// Register this patch (which starts pd)
 		PureDataController::getInstance()->RegisterPatch(this);
@@ -137,9 +135,7 @@ int PureDataConfigComponent::Initialize()
 		m_oscOut.Open();
 
 		// Open inbound socket and starts listening to it
-		m_oscIn.Open(); //Run(true);
-
-		m_initialized= true;
+		m_oscIn.Open();
 	}
 	catch (std::exception& e) {
 		getSpCoreRuntime()->LogMessage(ICoreRuntime::LOG_ERROR, e.what(),  GetTypeName());
@@ -150,10 +146,8 @@ int PureDataConfigComponent::Initialize()
 	return 0;	
 }
  
-void PureDataConfigComponent::Finish()
+void PureDataConfigComponent::DoFinish()
 {
-	if (!m_initialized) return;
-
 	// Save settings
 	SaveSettings();
 
@@ -165,7 +159,6 @@ void PureDataConfigComponent::Finish()
 	
 	try {
 		PureDataController::getInstance()->UnregisterPatch(this);
-		m_initialized= false;
 	}
 	catch (std::exception& e) {
 		assert (false);
