@@ -19,7 +19,7 @@
 // TODO: fix linux working issue, may require permanent pairing
 #include "spcore/component.h"
 #include "spcore/module.h"
-#include "spcore/libexports.h"
+#include "spcore/libimpexp.h"
 #include "spcore/basictypes.h"
 #include "mod_wiimotes.h"
 #include "wwiimotesconfiguration.h"
@@ -608,18 +608,16 @@ public:
 	static const char* getTypeName() { return "wiimotes_config"; };
 	virtual const char* GetTypeName() const { return WiimotesConfig::getTypeName(); };
 
-	virtual bool ProvidesExecThread() { return true; }
+	virtual bool ProvidesExecThread() const { return true; }
 
-	virtual bool NeedsInitialization() { return true; }
-
-	virtual int Initialize() {
+	virtual int DoInitialize() {
 		if (m_oPinStatus->GetNumComsumers()> 0) {
 			WiiuseThreadController::getInstance()->RegisterListener(*this, WiiuseThread::NONE, 0);
 		}
 		return 0;
 	}
 
-	virtual void Finish() {
+	virtual void DoFinish() {
 		WiiuseThreadController::getInstance()->UnregisterListener(*this);
 	}
 
@@ -755,11 +753,9 @@ public:
 	static const char* getTypeName() { return "wiimotes_input"; };
 	virtual const char* GetTypeName() const { return WiimotesInput::getTypeName(); };
 
-	virtual bool ProvidesExecThread() { return true; }
+	virtual bool ProvidesExecThread() const { return true; }
 
-	//virtual bool NeedsInitialization() { return true; }
-
-	virtual int Start() {
+	virtual int DoStart() {
 		unsigned int flags= 0, consumers= 0;
 
 		if (m_oPinWiimoteAcc->GetNumComsumers() > 0) {
@@ -789,7 +785,7 @@ public:
 		return 0;
 	}
 
-	virtual void Stop() {
+	virtual void DoStop() {
 		WiiuseThreadController::getInstance()->UnregisterListener(*this);
 	}
 
@@ -1187,7 +1183,7 @@ public:
 };
 
 static spcore::IModule* g_module= NULL;
-EXPORT_FUNCTION spcore::IModule* module_create_instance()
+SPEXPORT_FUNCTION spcore::IModule* module_create_instance()
 {
 	if (g_module== NULL) g_module= new WiiMotesModule();
 	return g_module;
