@@ -58,6 +58,7 @@
 
 using namespace spcore;
 using namespace sphost;
+using namespace std;
 
 // Flag to control whether more than one activity can be opened simultaneously.
 #define ALLOW_MULTIPLE_ACTIVITIES	0
@@ -364,13 +365,11 @@ void SitplusMainWindow::OpenConfigDialog(const char* component, const std::vecto
 	assert (panel);
 	dlg.AddSitplusPanel(panel);
 
-	if (compo->NeedsInitialization()) {
-		if (compo->Initialize()!= 0) {
-			wxMessageDialog msg(this, 
-				_("Configuration of ") + wxString(component, wxConvUTF8) + _(" failed.\nSee console for details"));
-			msg.ShowModal();
-			return;
-		}
+	if (compo->Initialize()!= 0) {
+		wxMessageDialog msg(this, 
+			_("Configuration of ") + wxString(component, wxConvUTF8) + _(" failed.\nSee console for details"));
+		msg.ShowModal();
+		return;
 	}
 
 	dlg.ShowModal();
