@@ -18,6 +18,7 @@
 /////////////////////////////////////////////////////////////////////////////
 
 #include "commoncomponents.h"
+#include "timecomponents.h"
 
 #include "spcore/basictypes.h"
 #include "spcore/pin.h"
@@ -821,6 +822,9 @@ CBasicTypesModule::CBasicTypesModule() {
 	RegisterComponentFactory(SmartPtr<IComponentFactory>(new SendMainAsyncFactory(), false));
 
 	RegisterComponentFactory(SmartPtr<IComponentFactory>(new PrintComponentFactory(), false));
+
+	// Time components
+	RegisterComponentFactory(SmartPtr<IComponentFactory>(new ChronoFactory(), false));
 }
 
 const char * CBasicTypesModule::GetName() const {
