@@ -32,6 +32,8 @@
 #include <boost/thread/mutex.hpp>
 #include <boost/thread/condition.hpp>
 
+using namespace std;
+
 namespace spcore {
 
 /**
@@ -713,9 +715,9 @@ public:
 		m_cr= getSpCoreRuntime();
 	}
 
-	virtual bool ProvidesExecThread() { return true; }
+	virtual bool ProvidesExecThread() const { return true; }
 
-	virtual int Start() {
+	virtual int DoStart() {
 		assert (m_cr->IsMainThread());
 
 		boost::mutex::scoped_lock (m_mutex);
@@ -724,7 +726,7 @@ public:
 		return 0;
 	}
    
-	virtual void Stop() {
+	virtual void DoStop() {
 		assert (m_cr->IsMainThread());
 
 		boost::mutex::scoped_lock lock(m_mutex);
@@ -858,14 +860,14 @@ public:
 		m_cr= getSpCoreRuntime();
 	}
 
-	virtual bool ProvidesExecThread() { return true; }
+	virtual bool ProvidesExecThread() const { return true; }
 
-	virtual int Start() {
+	virtual int DoStart() {
 		m_running= true;
 		return 0;
 	}
    
-	virtual void Stop() { m_running= false;	}
+	virtual void DoStop() { m_running= false; }
 
 	int OnSendReceived (const CTypeAny& msg) {
 		// If no listeners, do nothing
