@@ -45,6 +45,7 @@
 #include <errno.h>
 
 using namespace spcore;
+using namespace std;
 
 #ifdef WIN32
 #include <io.h>
@@ -508,7 +509,7 @@ private:
 	//
 	// Types
 	//
-	std::map<string,int> m_name2TypeID;
+	map<string,int> m_name2TypeID;
 	vector<ITypeFactory*> m_typeFactories;
 
 	//
@@ -865,7 +866,7 @@ static CCoreRuntime* g_coreRuntime= NULL;
 static boost::mutex g_spcore_mutex;
 static bool g_spcore_freed= false;
 
-EXPORT_FUNCTION ICoreRuntime* getSpCoreRuntime()
+SPEXPORT_FUNCTION ICoreRuntime* getSpCoreRuntime()
 {
 	assert (!g_spcore_freed);
 	boost::mutex::scoped_lock lock(g_spcore_mutex);
@@ -874,7 +875,7 @@ EXPORT_FUNCTION ICoreRuntime* getSpCoreRuntime()
 	return g_coreRuntime;
 }
 
-EXPORT_FUNCTION void freeSpCoreRuntime()
+SPEXPORT_FUNCTION void freeSpCoreRuntime()
 {
 //	boost::mutex::scoped_lock lock(g_spcore_mutex);
 
